- Added set_io_timeout(), which sets all the timeout-dependent
[rsync.git] / compat.c
index 84010215c17d7662b6190d472f33dbc45cabcb9b..be0bf473cb5e425682217ce3f1fb1967fccc1f0a 100644 (file)
--- a/compat.c
+++ b/compat.c
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
/* compatability routines for older rsync protocol versions */
 
 #include "rsync.h"
 
-extern int am_server;
-
-extern int csum_length;
-
-extern int preserve_links;
-extern int preserve_perms;
-extern int preserve_devices;
-extern int preserve_uid;
-extern int preserve_gid;
-extern int preserve_times;
-extern int always_checksum;
-extern int checksum_seed;
-
+int remote_protocol = 0;
 
-extern int remote_version;
 extern int verbose;
+extern int am_server;
+extern int am_sender;
+extern int inplace;
+extern int fuzzy_basis;
+extern int read_batch;
+extern int checksum_seed;
+extern int basis_dir_cnt;
+extern int protocol_version;
+extern char *dest_option;
 
 void setup_protocol(int f_out,int f_in)
 {
-  if (am_server) {
-    remote_version = read_int(f_in);
-    write_int(f_out,PROTOCOL_VERSION);
-    write_flush(f_out);
-  } else {
-    write_int(f_out,PROTOCOL_VERSION);
-    write_flush(f_out);
-    remote_version = read_int(f_in);
-  }
+       if (remote_protocol == 0) {
+               if (!read_batch)
+                       write_int(f_out, protocol_version);
+               remote_protocol = read_int(f_in);
+               if (protocol_version > remote_protocol)
+                       protocol_version = remote_protocol;
+       }
+       if (read_batch && remote_protocol > protocol_version) {
+               rprintf(FERROR, "The protocol version in the batch file is too new (%d > %d).\n",
+                       remote_protocol, protocol_version);
+               exit_cleanup(RERR_PROTOCOL);
+       }
 
-  if (remote_version < MIN_PROTOCOL_VERSION ||
-      remote_version > MAX_PROTOCOL_VERSION) {
-    rprintf(FERROR,"protocol version mismatch - is your shell clean?\n");
-    exit_cleanup(1);
-  }    
+       if (verbose > 3) {
+               rprintf(FINFO, "(%s) Protocol versions: remote=%d, negotiated=%d\n",
+                       am_server? "Server" : "Client", remote_protocol, protocol_version);
+       }
+       if (remote_protocol < MIN_PROTOCOL_VERSION
+        || remote_protocol > MAX_PROTOCOL_VERSION) {
+               rprintf(FERROR,"protocol version mismatch -- is your shell clean?\n");
+               rprintf(FERROR,"(see the rsync man page for an explanation)\n");
+               exit_cleanup(RERR_PROTOCOL);
+       }
+       if (remote_protocol < OLD_PROTOCOL_VERSION) {
+               rprintf(FINFO,"%s is very old version of rsync, upgrade recommended.\n",
+                       am_server? "Client" : "Server");
+       }
+       if (protocol_version < MIN_PROTOCOL_VERSION) {
+               rprintf(FERROR, "--protocol must be at least %d on the %s.\n",
+                       MIN_PROTOCOL_VERSION, am_server? "Server" : "Client");
+               exit_cleanup(RERR_PROTOCOL);
+       }
+       if (protocol_version > PROTOCOL_VERSION) {
+               rprintf(FERROR, "--protocol must be no more than %d on the %s.\n",
+                       PROTOCOL_VERSION, am_server? "Server" : "Client");
+               exit_cleanup(RERR_PROTOCOL);
+       }
 
-  if (verbose > 2)
-         rprintf(FINFO, "local_version=%d remote_version=%d\n",
-                 PROTOCOL_VERSION, remote_version);
+       if (fuzzy_basis && protocol_version < 29) {
+               rprintf(FERROR,
+                       "--fuzzy requires protocol 29 or higher (negotiated %d).\n",
+                       protocol_version);
+               exit_cleanup(RERR_PROTOCOL);
+       }
 
-  if (remote_version >= 12) {
-    if (am_server) {
-      checksum_seed = time(NULL);
-      write_int(f_out,checksum_seed);
-    } else {
-      checksum_seed = read_int(f_in);
-    }
-  }
+       if (basis_dir_cnt && inplace && protocol_version < 29) {
+               rprintf(FERROR,
+                       "%s with --inplace requires protocol 29 or higher (negotiated %d).\n",
+                       dest_option, protocol_version);
+               exit_cleanup(RERR_PROTOCOL);
+       }
 
-  checksum_init();
-}
+       if (basis_dir_cnt > 1 && protocol_version < 29) {
+               rprintf(FERROR,
+                       "Multiple %s options requires protocol 29 or higher (negotiated %d).\n",
+                       dest_option, protocol_version);
+               exit_cleanup(RERR_PROTOCOL);
+       }
 
+       if (am_server) {
+               if (!checksum_seed)
+                       checksum_seed = time(NULL);
+               write_int(f_out, checksum_seed);
+       } else {
+               checksum_seed = read_int(f_in);
+       }
+}
@@
 */
 
-/* compatability routines for older rsync protocol versions */
+/**
+ * @file compat.c
+ *
+ * Compatibility routines for older rsync protocol versions.
+ **/
 
 #include "rsync.h"